<?php
/**
 * RecordingRule encapsulates a MythTV recording rule.
 *
 * Each instantiation of RecordingRule represents a single row from the
 * record table of the MythTV database.
 *
 * @version $Id: RecordingRule.php,v 1.7 2006/04/05 22:39:17 dbattams Exp $
 * @author Derek Battams <derek@battams.ca>
 * @package MRE
 * @copyright 2006 GPL v2
 */

/**#@+
 * Required include
 */
require_once(APP_ROOT . '/class/SysConf.php');
require_once(APP_ROOT . '/class/Channel.php');
require_once(APP_ROOT . '/class/MythClient.php');
/**#@-*/

/**
 * Encapsulates a MythTV recording rule.
 *
 * @package MRE
 */
class RecordingRule
{
   /**#@+
    * From libs/libmythtv/recordingtypes.h
    */

   /**
    * Single record rule
    */
   const RECTYPE_SINGLE   = 1;

   /**
    * Override record rule
    */
   const RECTYPE_OVERRIDE = 7;

   /**
    * A recording rule with no search type
    */
   const SEARCHTYPE_NONE  = 0;
   /**#@-*/

   /**
    * @var array A hash representing a row from the record table in MythTV
    */
   private $_row;

   /**
    * @var bool Flag denoting if the DB row was changed and therefore needs to be rewritten
    */
   private $_changed;

   /**
    * Class constructor
    *
    * @param integer An ID uniquely identifying a MythTV recording rule
    */
   public function __construct($data)
   {
      if(is_numeric($data))
         $this->loadRuleFromDB($data);
      else
         trigger_error("Invalid arguments!", E_USER_ERROR);
      $this->setChanged(FALSE);
      return;
   }

   /**
    * Copy constructor
    *
    * All copies of RecordingRule objects are actually override rules to
    * be used to override the source rule.  The __clone method does some
    * object manipulation required to turn the object into an override object.
    */
   public function __clone()
   {
      $this->setVal('recordid', NULL);
      $this->setVal('type', self::RECTYPE_OVERRIDE);
      $this->setVal('search', self::SEARCHTYPE_NONE);
      $this->setVal('findid', 0);
      return;
   }

   /**
    * Class destructor.
    *
    * The destructor will write the row back to the database if it was marked
    * as changed.
    */ 
   public function __destruct()
   {
      $conf = SysConf::getInstance();
      if($this->getVal('type') != self::RECTYPE_SINGLE && $this->getVal('type') != self::RECTYPE_OVERRIDE)
      {
         if($conf->getOpt('verbose') >= SysConf::DEBUG)
            echo "Skipping write of recording rule, not a valid type!\n";
         return;
      }
      if(!$this->isChanged())
      {
         if($conf->getOpt('verbose') >= SysConf::DEBUG)
            echo "Skipping write of recording rule, nothing was changed!\n";
         return;
      }

      $db = $conf->getDB();
      $qry = "REPLACE INTO record SET ";
      $cols = array();
      $vals = array();
      foreach($this->_row as $col => $val)
      {
         $cols[] =  "$col = ?";
         $vals[] = $val;
      }
      $qry .= implode(',', $cols);
      $qh = $db->prepare($qry);
      if(DB::isError($qh))
         trigger_error("Can't prepare query!", E_USER_ERROR);
      $res = $db->execute($qh, $vals);
      if(DB::isError($res))
      {
         echo $res->getDebugInfo(). "\n";
         trigger_error("Update of recording rule failed!", E_USER_ERROR);
      }
      $db->freePrepared($qh);
      $client = new MythClient();
      $client->reschedule();
      return;
   }

   /**#@+
    * Setters and getters
    */
   public function getVal($var)
   {
      if(isset($this->_row[$var]))
         return $this->_row[$var];
      trigger_error("Invalid property!", E_USER_WARNING);
      return NULL;
   }

   public function setVal($var, $val)
   {
      if(isset($this->_row[$var]))
         $this->_row[$var] = $val;
      else
         trigger_error("Invalid property!", E_USER_WARNING);
      return;
   }

   public function setChanged($bool)
   {
      $this->_changed = (bool) $bool;
      return;
   }

   public function isChanged()
   {
      return $this->_changed;
   }
   /**#@-*/

   /**
    * Given an id, load the recording rule from the MythTV database
    *
    * @param integer A unique id of a MythTV recording rule
    */
   protected function loadRuleFromDB($recordid)
   {
      $conf = SysConf::getInstance();
      $db = $conf->getDB();
      $db->setFetchMode(DB_FETCHMODE_ASSOC);
      $qry = "SELECT * FROM record WHERE recordid = $recordid";
      $res = $db->query($qry);
      if(DB::isError($res))
      {
         if($conf->getOpt('verbose') >= SysConf::DEBUG)
            echo $res->getDebugInfo();
         else if($conf->getOpt('verbose') >= SysConf::ERROR)
            echo $res->getMessage();
         trigger_error("Can't find required recording rule!", E_USER_ERROR);
      }
      if($res->numRows() != 1)
      {
         if($conf->getOpt('verbose') >= SysConf::DEBUG)
            echo "Query: $qry\n";
         trigger_error("Unexpected number of rows returned!", E_USER_ERROR);
      }
      $this->_row = $res->fetchRow();
      $res->free();
      return;
   }

   /**
    * Given a parent rule, clone it as an override for the parent.
    *
    * This method takes a parent rule and fills itself with the same data where
    * needed and edits the fields necessary to make this object a valid override
    * rule for the parent.  It's override rules that allow us to modify the
    * recording time of MythTV recordings.
    *
    * @param Recording The parent rule we're going to be overriding
    */
   public function fillOverride(Recording $rec)
   {
      $this->setVal('chanid', $rec->getVal('chanid'));

      $tmp = explode(' ', $rec->getVal('progstart'));
      $this->setVal('starttime', $tmp[1]);
      $this->setVal('startdate', $tmp[0]);

      $tmp = explode(' ', $rec->getVal('progend'));
      $this->setVal('endtime', $tmp[1]);
      $this->setVal('enddate', $tmp[0]);

      $this->setVal('title', $rec->getVal('title'));
      $this->setVal('subtitle', $rec->getVal('subtitle'));
      $this->setVal('description', $rec->getVal('description'));
      $this->setVal('category', $rec->getVal('category'));
      $this->setVal('seriesid', $rec->getVal('seriesid'));
      $this->setVal('programid', $rec->getVal('programid'));
      $this->setVal('transcoder', $rec->getVal('transcoder'));

      $chan = new Channel($rec->getVal('chanid'));
      $this->setVal('station', $chan->getCallsign());
      return;
   }
}

?>
